%% PFrameMatlab (PFrameMatlab.m)
%  Program for the Linear Static Analysis of Plane Frames with Matlab
%  ------------------------------------------------------------------------
%  v1.2, 23 November 2015
%  (c) Copyright 2008-2015
%  Vagelis Plevris, Apostolis Vrakas, Manolis Papadrakakis
%  All rights reserved
%  email:
%  vplevris@gmail.com, apostolis.vrakas@gmail.com, mpapadra@central.ntua.gr
%  Institute of Structural Analysis and Seismic Research
%  School of Civil Engineering, National Technical University of Athens
%  ------------------------------------------------------------------------
%  Files that belong to the program:
%  1. PFrameMatlab.m (this file)
%  2. PFrameDataInput.m
%  3. PFrameAssignElementProperties.m
%  4. PFrameElementLength.m
%  5. PFrameT1ElementStiffness.m
%  6. PFrameT2ElementStiffness.m
%  7. PFrameT3ElementStiffness.m
%  8. PFrameT4ElementStiffness.m
%  9. PFrameElementTransformation.m
% 10. PFrameDataOutput.m
% 11. PFrameDrawModel.m
%
% An input file is also needed containing the model data
% 2 example input files (model1.dat, model2.dat) are included in the package
% When run for model1.dat, the program generates the output text file model1.dat.out
% When run for model2.dat, the program generates the output text file model2.dat.out


%% Start main program PFrameMatlab.m
clear %Clear memory (workspace)
clc %Clear command window
format long; %Show results with many decimals


%% Get model data and draw model
PFrameDataInput;
tic %Initialize timer for analysis
PFrameDrawModel %Draw model
NumDOFs = 3*NumNodes; %Total Degrees Of Freedom for whole model

%% Form structure's global stiffness matrix
%  KGlobalStructure [NumDOFs x NumDOFs]
KGlobalStructure=zeros(NumDOFs); %Initialize KGlobalStructure putting zeros in it
for n=1:NumElements %Scan all elements
	PFrameAssignElementProperties; %Get properties for n element
    %Form matrices
    switch ElementProperties(n,5) %Form local stiffness matrix (KLocal) for n element [6x6] based on element type
        case 1 % Beam -------- with no hinges (P2 type)
            KLocal = PFrameT1ElementStiffness(E,A,I,L); 
        case 2 % Beam o------- with hinge at start i (P2' type, hinge at start i)
            KLocal = PFrameT2ElementStiffness(E,A,I,L); 
        case 3 % Beam -------o with hinge at end j (P2' type, hinge at end j)
            KLocal = PFrameT3ElementStiffness(E,A,I,L);
        case 4 % Beam o------o with hinges at both ends (P1 Type, hinges at start i and end j)
            KLocal = PFrameT4ElementStiffness(E,A,I,L);
    end
	Transform = PFrameElementTransformation(x1,y1,x2,y2); %Form transformation matrix (Transform) for n element [6x6]
	KGlobal = (Transform')*KLocal*Transform ; %Form global stiffness matrix (KGlobal) for n element [6x6]
	DOFi = 3*Node_i-2 ; %x-Degree OF Freedom for start node (i)
	DOFj = 3*Node_j-2 ; %x-Degree OF Freedom for end node (j)
    
	%Add the element's global stiffness matrix [6x6] to the appropriate position in
	%the structure's global stiffness matrix [NumNodes*3 x NumNodes*3]
	KGlobalStructure(DOFi:DOFi+2,DOFi:DOFi+2)=KGlobalStructure(DOFi:DOFi+2,DOFi:DOFi+2)+KGlobal(1:3,1:3);
	KGlobalStructure(DOFj:DOFj+2,DOFj:DOFj+2)=KGlobalStructure(DOFj:DOFj+2,DOFj:DOFj+2)+KGlobal(4:6,4:6);
	KGlobalStructure(DOFi:DOFi+2,DOFj:DOFj+2)=KGlobalStructure(DOFi:DOFi+2,DOFj:DOFj+2)+KGlobal(1:3,4:6);
	KGlobalStructure(DOFj:DOFj+2,DOFi:DOFi+2)=KGlobalStructure(DOFj:DOFj+2,DOFi:DOFi+2)+KGlobal(4:6,1:3);
end


%% Form Rearrangement matrix V
ZeroDisplacements = 0; %Set known (zero) DOF displacements to zero
UnknownDisplacements = 0; %Set unknown DOF displacements to zero
for n=1:(NumDOFs) %Scan all DOFs
	node = fix((n-1)/3)+1; %Node where the DOF belongs to
	DOF_ID = n-3*(node-1); %Assign 1, 2, 3 for x-displacement, y-displacement or z-rotation, respectively
	if BoundaryConditions(node,DOF_ID)==1 %In case of constrained DOF
        ZeroDisplacements = ZeroDisplacements + 1; %Count known DOFs
        V1order(ZeroDisplacements)=n; %order known DOFs assending
	else UnknownDisplacements =UnknownDisplacements + 1; %In case of unconstrained DOF, Count unknown DOFs
        V2order(UnknownDisplacements)=n; %order unknown DOFs assending
	end
end

if UnknownDisplacements==0 Vorder = V1order;
else Vorder = [V2order V1order]; %Put together to form Vorder
end
 
V = zeros(NumDOFs); %Initialize V putting zeros in it
for n=1:NumDOFs; V(n,Vorder(n))=1; end; %Form rearrangement matrix by putting ones into the appropriate positions
    

%% Form rearranged structure's global stiffness matrix (Km)
Km = V*KGlobalStructure*(V');
Kff = Km(1:UnknownDisplacements,1:UnknownDisplacements);
Ksf = Km((UnknownDisplacements+1):NumDOFs,1:UnknownDisplacements);


%% Form Nodal loads array LoadVec
LoadVec = zeros(NumDOFs,1); %Initialize LoadVec putting zeros in it
for n=1:NumNodes; LoadVec((3*n-2):(3*n))=NodalLoads(n,:); end

LoadVecRearranged = V*LoadVec; %Rearranged load vector


%% Calculate unknown displacements
Pf = LoadVecRearranged(1:UnknownDisplacements);
Df = (inv(Kff))*Pf; %Calculate unknown displacements (global system)

DisplVecRearranged = [Df ; zeros(ZeroDisplacements,1)]; %rearranged vector of displacements (global system)
DisplacementsGlobal = V'*DisplVecRearranged; %Vector of displacements (global system)


%% Calculate constraint reactions
Rinitial = Ksf*Df;
ConstraintReactions = Rinitial-LoadVecRearranged((UnknownDisplacements+1):NumDOFs); %In case there are forces directly on constrained DOFs


%% Calculate element forces in global system
ElementALLLocalForces = zeros(NumElements,6); %Initialize ElementALLLocalForces putting zeros in it
for n=1:NumElements %Scan all elements
	PFrameAssignElementProperties  %Get properties for n element
	switch ElementProperties(n,5) %Form local stiffness matrix (KLocal) for n element [6x6] based on element type
        case 1 % Beam -------- with no hinges (P2 type)
            KLocal = PFrameT1ElementStiffness(E,A,I,L); 
        case 2 % Beam o------- with hinge at start i (P2' type, hinge at start i)
            KLocal = PFrameT2ElementStiffness(E,A,I,L); 
        case 3 % Beam -------o with hinge at end j (P2' type, hinge at end j)
            KLocal = PFrameT3ElementStiffness(E,A,I,L);
        case 4 % Beam o------o with hinges at both ends (P1 Type, hinges at start i and end j)
            KLocal = PFrameT4ElementStiffness(E,A,I,L);
    end
    Transform = PFrameElementTransformation(x1,y1,x2,y2); %n element transformation matrix [6x6]
	DOFi = 3*Node_i-2 ; %x-Degree OF Freedom for start node (i) of n element
	DOFj = 3*Node_j-2 ; %x-Degree OF Freedom for end node (j) of n element
    
    ElementnGlobalDisplacements = [DisplacementsGlobal(3*Node_i-2:3*Node_i) ; DisplacementsGlobal(3*Node_j-2:3*Node_j)]; %n element displacements in global system [6x1]
    ElementnLocalDisplacements = Transform*ElementnGlobalDisplacements; %n element displacements in local system [6x1]
    ElementnLocalForces = KLocal*ElementnLocalDisplacements; %n element forces in local system [6x1]
	ElementALLLocalForces(n,:)=ElementnLocalForces; %Add element contribution to the local forces matrix for all elements [NumElements x 6]
end


%% Do some postprocessing and export analysis results
%Convert displacements to format [NumNodes x 3] where each row is one node
for n=1:NumNodes;
    NodeDisplacementsGlobal(n,1:3)=DisplacementsGlobal(3*n-2:3*n);
end

%Convert constraint reactions to format: [Node DOF Reaction]
nodeIDReaction = fix((V1order' - 1) / 3) + 1; %Node where the DOF belongs to
DOFIDReaction = V1order' - 3 * (nodeIDReaction - 1); %Assign 1, 2, 3 for x-displacement, y-displacement or z-rotation, respectively
NodeDOFReaction = [nodeIDReaction DOFIDReaction ConstraintReactions]; %Put them all together

PFrameDataOutput; %Export analysis results
toc %terminate timer and report elapsed time

% End of file -------------------------------------------------------------